@@ -1,6 +1,6 @@
 /* Associativity information.
 
-   Copyright (C) 2002, 2006, 2009, 2010 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2006, 2008-2013 Free Software Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
 
@@ -23,10 +23,11 @@
 /* Associativity values for tokens and rules.  */
 typedef enum
 {
-  undef_assoc,
-  right_assoc,
-  left_assoc,
-  non_assoc
+  undef_assoc,     /** Not defined. */
+  right_assoc,     /** %right */
+  left_assoc,      /** %left */
+  non_assoc,       /** %nonassoc */
+  precedence_assoc /** %precedence */
 } assoc;
 
 char const *assoc_to_string (assoc a);
