@@ -1,6 +1,6 @@
 /* Match rules with nonterminals for bison,
 
-   Copyright (C) 1984, 1989, 2000-2003, 2005, 2009-2011 Free Software
+   Copyright (C) 1984, 1989, 2000-2003, 2005, 2009-2013 Free Software
    Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
@@ -47,12 +47,13 @@ print_derives (void)
   for (i = ntokens; i < nsyms; i++)
     {
       rule **rp;
-      fprintf (stderr, "\t%s derives\n", symbols[i]->tag);
+      fprintf (stderr, "  %s derives\n", symbols[i]->tag);
       for (rp = derives[i - ntokens]; *rp; ++rp)
-	{
-	  fprintf (stderr, "\t\t%3d ", (*rp)->user_number);
-	  rule_rhs_print (*rp, stderr);
-	}
+        {
+          fprintf (stderr, "    %3d ", (*rp)->user_number);
+          rule_rhs_print (*rp, stderr);
+          fprintf (stderr, "\n");
+        }
     }
 
   fputs ("\n\n", stderr);
@@ -96,10 +97,10 @@ derives_compute (void)
       rule_list *p = dset[i - ntokens];
       derives[i - ntokens] = q;
       while (p)
-	{
-	  *q++ = p->value;
-	  p = p->next;
-	}
+        {
+          *q++ = p->value;
+          p = p->next;
+        }
       *q++ = NULL;
     }
 
