@@ -1,6 +1,6 @@
 /* Output the generated parsing program for bison,
 
-   Copyright (C) 2000-2003, 2006-2007, 2009-2011 Free Software
+   Copyright (C) 2000-2003, 2006-2007, 2009-2013 Free Software
    Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
@@ -23,6 +23,8 @@
 
 /* Output the parsing tables and the parser code to FTABLE.  */
 void output (void);
-char const *compute_pkgdatadir (void);
+
+/* Where our data files are installed.  */
+char const *pkgdatadir (void);
 
 #endif /* !OUTPUT_H_ */
