@@ -1,7 +1,7 @@
 /* Grammar reduction for Bison.
 
-   Copyright (C) 2000, 2001, 2002, 2007, 2009, 2010 Free Software
-   Foundation, Inc.
+   Copyright (C) 2000-2002, 2007, 2009-2013 Free Software Foundation,
+   Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
 
